/** Sends a message to 'jms/Samples/Q1'
    and reads it back again.

    Requires a QueueConnectionFactory named 'jms/Samples/QCF1' and
    a Queue named 'jms/Samples/Q1' to be available via JNDI.

    Note that this EJB is intended only to illustrate the use of
    JMS.
*/

package sample1;

import java.rmi.RemoteException; 
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.jms.*;
import javax.naming.*;

public class Sample1EJB implements SessionBean
{

  // Put message to queue
  public String putMessage(String text)
    throws JMSException, Exception
  {
    String result = null;      

    QueueSession session = null;
    QueueSender sender = null;

    QueueConnectionFactory qcf = null;
    QueueConnection jmsCon = null;

    System.out.println("request to put message '"+text+"'");

    try {

      // Obtain the connection factory from JNDI
      InitialContext ic = new InitialContext();
      qcf = (QueueConnectionFactory) ic.lookup("jms/Samples/QCF1");
 
      // Create a connection
      jmsCon = qcf.createQueueConnection();
      jmsCon.start();
      // Create a session. Note that the parameters are ignored if
      // the connection factory is a WebSphere Specific one (as per the JMS
      // spec' for an XAQueueConnection).
      session = jmsCon.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

      // Obtain the Destination object from JNDI
      Queue ioQueue = (Queue) ic.lookup("jms/Samples/Q1");

      // Create a QueueSender
      System.err.println("creating queue sender");
      sender = session.createSender(ioQueue);
      
      // Create a message to send to the queue...
      TextMessage message = session.createTextMessage(text);

      // ...and send it
      System.out.println("sending message");
      sender.send(message);

      // store the message id for the return value
      result = message.getJMSMessageID();

      // Close the connection (close calls will cascade to other objects)
      jmsCon.close();
      jmsCon = null;

      System.out.println("done");

    } catch (JMSException je) {
      System.out.println("failed with "+je);
      Exception le = je.getLinkedException();
      if (le != null) System.out.println("linked exception "+le);

      // Pass the exception back to the caller
      throw je;

    } catch (Exception e) {
      System.out.println("failed with "+e);

      // Pass the exception back to the caller
      throw e;

    } finally {
      // Ensure that the Connection always gets closed
      if (jmsCon != null) {
        try {
          jmsCon.close();
        } catch (JMSException je) {
        }
      }
    }

    return result;

  }


  // Get message from queue
  public String getMessage(String messageID)
    throws JMSException, Exception
  {
    QueueSession session = null;
    QueueSender sender = null;

    QueueConnectionFactory qcf = null;
    QueueConnection jmsCon = null;

    String body = null;

    System.out.println("request to get message '"+messageID+"'");

    try {

      // Obtain connection factory from JNDI
      InitialContext ic = new InitialContext();
      qcf = (QueueConnectionFactory) ic.lookup("jms/Samples/QCF1");

      // Create connection
      jmsCon = qcf.createQueueConnection();
      jmsCon.start();

      // Create session
      session = jmsCon.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

      // Lookup queue
      Queue ioQueue = (Queue) ic.lookup("jms/Samples/Q1");

      // Create a receiver
      String selector = "JMSMessageID = '"+messageID+"'";
      QueueReceiver queueReceiver = session.createReceiver(ioQueue, selector);

      System.out.println("getting message");
      Message inMessage = queueReceiver.receive(5000);

      if (inMessage == null) {
        System.out.println("no message found");

      } else if (inMessage instanceof TextMessage) {
        body = ((TextMessage)inMessage).getText();
        System.out.println("body is >"+body+"<");

      } else {
        System.out.println("message not a TextMessage");
        System.out.println(inMessage);
      }

      System.out.println("closing receiver");
      queueReceiver.close();

      System.out.println("closing connection");
      jmsCon.close();
      jmsCon = null;

    } catch (JMSException je) {
      System.out.println("failed with "+je);
      Exception le = je.getLinkedException();
      if (le != null) System.out.println("linked exception: "+le);

      // pass exception back to client
      throw je;

    } catch (Exception e) {
      System.out.println("failed with "+e);

      // pass exception back to client
      throw e;
    }

    return body;

  }


  public void ejbCreate()
  {
    System.out.println("Sample1EJB.ejbCreate()");
  }

  public void ejbRemove()
  {
    System.out.println("Sample1EJB.ejbRemove()");
  }

  public void ejbActivate()
  {
    System.out.println("Sample1EJB.ejbActivate()");
  }

  public void ejbPassivate()
  {
    System.out.println("Sample1EJB.ejbPassivate()");
  }

  public void setSessionContext(SessionContext sc) {}

}
  
